<?php

namespace Nwb\UrlToPdf;

class UrlToPdf
{
    private string $apiUrl = 'https://url-to-pdf.kub.nwb.fr';

    /**
     * @var string|null
     */
    private ?string $accessKey;
    private array $seenResources = [];

    /**
     * UrlToPdf constructor.
     * @param array $options Liste des options de configuration
     * @throws \Exception
     */
    public function __construct(array $options = [])
    {
        $key = $options['accessKey'] ?? $_ENV['URL_TO_PDF_ACCESS_KEY'] ?? null;
        if (!$key) {
            throw new \Exception('Missing access key');
        }
        $this->accessKey = $key;
        $this->apiUrl = $options['apiUrl'] ?? $_ENV['URL_TO_PDF_APIURL'] ?? $this->apiUrl;
    }

    /**
     * Requête curl vers l'API
     * 
     * @param string $url contient la fin de l'url pour appeler la bonne méthode de l'API (ex : /get-pdf)
     * @param array $options Liste des options de configuration, notamment $options['url'] l'url vers la page html à convertir en PDF
     * @return false|resource contenu du PDF
     * @throws \Exception si la page est vide
     * @throws \Exception si le code HTTP n'est pas 200
     * @throws \Exception si le contenu de la réponse n'est pas un PDF
     * @throws \Exception si le contenu de la réponse est vide
     */
    private function request(string $url, array $options)
    {
        if (empty($options['url'])) {
            throw new \Exception('Missing url');
        }

        // On construit l'url de l'HTML à transmettre à l'API si authentification basique au server (ex: https://username:password@url)
        if (!empty($options['auth']['username']) && !empty($options['auth']['password'])) {
            $username = $options['auth']['username'];
            $password = $options['auth']['password'];
            $options['url'] = str_replace('://', '://' . $username . ':' . $password . '@', $options['url']);
        }

        $urlMustHave200 = $options['urlMustHave200'] ?? false;

        // --- Toujours vérifier que la page HTML n'est pas vide, comme avant ---
        // On fait un cURL GET complet (pas partiel) pour récupérer le HTML.
        $ch1 = curl_init($options['url']);
        curl_setopt($ch1, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch1, CURLOPT_FOLLOWLOCATION, true);
        $pageContent = curl_exec($ch1);

        if ($pageContent === false) {
            throw new \Exception("Erreur cURL lors de la récupération du fichier html : " . curl_error($ch1));
        }

        if (empty($pageContent)) {
            curl_close($ch1);
            throw new \Exception("La page html récupérée n'a pas de contenu.");
        }

        $httpCode = curl_getinfo($ch1, CURLINFO_HTTP_CODE);
        $contentType = curl_getinfo($ch1, CURLINFO_CONTENT_TYPE);
        curl_close($ch1);

        if ($urlMustHave200) {
            // Si on a besoin de 200, on vérifie
            if ($httpCode !== 200) {
                throw new \Exception('Error: ' . $httpCode);
            }

            if (strpos($contentType, 'html') === false) {
                throw new \Exception("Le contenu récupéré n'est pas de type HTML.");
            }
        }

        // Maintenant on appelle l'API
        $ch = curl_init();
        $headers = [
            'Content-Type: application/json',
            'ACCESS-KEY: ' . $this->accessKey,
        ];

        curl_setopt($ch, CURLOPT_URL, $this->apiUrl . $url);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($options));
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        $response = curl_exec($ch);

        if ($response === false) {
            throw new \Exception("Erreur cURL : " . curl_error($ch));
        }

        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        $contentType = curl_getinfo($ch, CURLINFO_CONTENT_TYPE);
        curl_close($ch);

        if ($httpCode !== 200) {
            if (strpos($contentType, 'application/json') !== false) {
                $errorText = json_decode($response, true);
                if (!empty($errorText)) {
                    $message = $errorText['message'] ?? ($errorText['message'] ?? print_r($errorText, true));
                    throw new \Exception('Erreur de l\'API : ' . $message);
                }
            } else {
                if ($response) {
                    throw new \Exception('Erreur cURL : ' . $response);
                }
            }
            throw new \Exception('Code d\'erreur http : ' . $httpCode);
        }

        if (empty($response) || substr($response, 0, 4) !== "%PDF") {
            throw new \Exception('La réponse retournée est vide ou n\'est pas un PDF.');
        }

        return $response;
    }

    /**
     * Enregistre le contenu dans un fichier sur le disque.
     *
     * @param string $content Contenu à enregistrer.
     * @param string $destination Chemin de destination du fichier.
     * @return bool
     */
    public function saveTo(string $content, string $destination): bool
    {
        return file_put_contents($destination, $content) !== false;
    }

    /**
     * Retourne un PDF à partir d'une URL et des options
     * 
     * @param array $options Liste des options de configuration
     * @return false|resource contenu du PDF
     */
    public function getPdf(?array $options)
    {
        $options = $options ?? [
            'url' => 'https://normandiewebschool.fr/cursus/bachelor-developpement-web/',
            'urlMustHave200' => false,
            'loadingOptions' => ['waitUntil' => 'networkidle2'],
            'pdfOptions' => ['format' => 'a4']
        ];

        return $this->request('/get-pdf', $options);
    }

    public function getPdfFromHtml(
        string $htmlContent,
        string $projectRootPath,
        string $webPath,
        string $webPathUrl,
        array $loadingOptions = [],
        array $pdfOptions = [],
        array $authOptions = [],
        bool $isTestMode = false,
        $domaineException = []
    ): string {
        if (!$htmlContent) {
            throw new \Exception("Le contenu HTML est vide.");
        }

        if (!is_dir($webPath)) {
            throw new \Exception("Le répertoire web n'existe pas : " . $webPath);
        }

        if (!$webPathUrl || strlen($webPathUrl) < 8) {
            throw new \Exception("Le chemin web n'est pas valide : " . $webPathUrl);
        }

        if (substr($webPath, -1) !== '/') {
            $webPath .= '/';
        }

        if (substr($webPathUrl, -1) !== '/') {
            $webPathUrl .= '/';
        }

        $webPathUrl = trim($webPathUrl);

        // Vérification de l'url formée
        if (!$this->isValidUrl($webPathUrl)) {
            throw new \Exception("L'url n'est pas valide pour : " . $webPathUrl);
        }

        $webPathFinal = $webPath;

        $name_dir = md5($htmlContent);
        $tempWebPath = $webPath . $name_dir . '/';
        $tempWebPathUrl = $webPathUrl . $name_dir . '/';

        $pdfOptions = $pdfOptions ?? ['format' => 'a4'];
        $pdfOptions['noBreakTableRows'] = $pdfOptions['noBreakTableRows'] ?? true;
        $pdfOptions['urlMustHave200'] = $pdfOptions['urlMustHave200'] ?? false;

        $cleanHtml = $this->getCleanHtml($htmlContent, $projectRootPath, $tempWebPath, $tempWebPathUrl, $pdfOptions, $isTestMode, $domaineException);

        if (!is_dir($tempWebPath)) {
            if (!mkdir($tempWebPath, 0777, true)) {
                throw new \Exception("Le répertoire web n'a pas pu être créé.");
            }
        }

        $fileName = 'pdf.html';
        $filePath = $tempWebPath . $fileName;
        file_put_contents($filePath, $cleanHtml);
        chmod($filePath, 0777);

        $loadingOptions = $loadingOptions ?? ['waitUntil' => 'networkidle2', 'timeout' => 30000];

        if (!$isTestMode) {
            $pdf = $this->getPdf([
                'url' => $tempWebPathUrl . $fileName,
                'urlMustHave200' => $pdfOptions['urlMustHave200'],
                'loadingOptions' => $loadingOptions,
                'pdfOptions' => $pdfOptions,
                'auth' => $authOptions
            ]);
        } else {
            file_put_contents($tempWebPath . 'TEST_' . $fileName, $cleanHtml);
            $encodedHtml = base64_encode($cleanHtml);
            $encodedHtml = substr($encodedHtml, 0, 3000);
            $pdf = $this->getPdf([
                'url' => 'https://n8n.kub.nwb.fr/webhook/Base64ToHtml?htmlB64=' . $encodedHtml,
                'pdfOptions' => $pdfOptions,
                // Le test mode ne nécessite pas urlMustHave200
            ]);
        }

        file_put_contents($filePath, $pdf);
        rename($filePath, $tempWebPath . 'pdf.pdf');
        $filePath = $tempWebPath . 'pdf.pdf';

        $hashFileName = md5_file($filePath);
        $filePathReturn = $webPathFinal . $hashFileName . '.pdf';
        copy($filePath, $filePathReturn);

        if (!$isTestMode) {
            $this->deleteTempImages($tempWebPath);
        }

        return $filePathReturn;
    }

    private function extractAndCheckBackgroundUrls(string $html): array
    {
        $regex = '/background(?:-image)?:\s*url\((["\']?)(.*?)\1\)/i';
        $uniqueUrls = [];
        if (preg_match_all($regex, $html, $matches)) {
            foreach ($matches[2] as $url) {
                $url = trim($url);
                if (!in_array($url, $uniqueUrls)) {
                    $uniqueUrls[] = $url;
                }
            }
        }
        return $uniqueUrls;
    }

    private function getImageInfo($url)
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_NOBODY, true);
        curl_setopt($ch, CURLOPT_HEADER, true);
        curl_setopt($ch, CURLOPT_TIMEOUT, 10);
        curl_exec($ch);

        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        $contentType = null;

        if ($httpCode == 200) {
            $header = curl_getinfo($ch, CURLINFO_CONTENT_TYPE);
            if ($header) {
                $contentType = $header;
            }
        }

        curl_close($ch);

        if ($httpCode != 200 || !$contentType) {
            throw new \Exception("Impossible d'accéder à cette image : " . $url);
        }

        return [
            'http_code' => $httpCode,
            'content_type' => $contentType
        ];
    }

    private function getFileName($filePath)
    {
        $path = parse_url($filePath, PHP_URL_PATH);
        return basename($path);
    }

    private function getExtensionFromContentType($contentType, $nameFile = null)
    {
        $mimeTypes = [
            'image/jpeg' => 'jpg',
            'image/png' => 'png',
            'image/gif' => 'gif',
            'image/bmp' => 'bmp',
            'image/webp' => 'webp',
            'image/svg' => 'svg'
        ];

        if (!empty($mimeTypes[$contentType])) {
            return $mimeTypes[$contentType];
        } elseif (!empty($nameFile)) {
            $extension = pathinfo($nameFile, PATHINFO_EXTENSION);
            if ($extension) {
                return strtolower($extension);
            }
        }
        return null;
    }

    private function urlExists($url)
    {
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_NOBODY, true);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_TIMEOUT, 10);
        curl_setopt($ch, CURLOPT_HEADER, true);
        curl_exec($ch);
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);
        return $httpCode == 200;
    }

    private function getDomainFromUrl($url)
    {
        $parsedUrl = parse_url($url);
        return $parsedUrl['host'] ?? null;
    }

    private function isDomainAllowed($url, $allowedDomains)
    {
        $domain = $this->getDomainFromUrl($url);
        if ($domain) {
            return in_array($domain, $allowedDomains);
        }
        return false;
    }

    /**
     * Cette fonction vérifie la validité d'une URL, y compris les cas sans TLD ou avec localhost.
     * @param string $url L'URL à valider
     * @return bool true si l'URL est valide, false sinon
     */
    public function isValidUrl($url)
    {
        // Le test isValidUrlTest veut que http://localhost:8080 soit valable.
        $formatPath = "/^(https?|ftp|s3|file|gopher|ws|wss):\/\/(localhost|[\w\.-]+(\.[a-z\.]{2,6})?)(:[0-9]+)?([\/\w\.\-]*)*(\?[\/\w\.\-=&%]*)?\/?(?:#[\w\-]*)?$/i";
        return (bool)preg_match($formatPath, $url);
    }

    /**
     * Retourne le html passé en paramètre avec les path des fichiers accessibles pour l'API Url to PDF
     * 
     * @param string $htmlIn
     * @param string $projectRootPath (ex: /var/www/html/)
     * @param string $webPath (ex: /var/www/html/public-data/tmp/url_to_pdf/)
     * @param string $webPathUrl (ex: https://www.nwb.fr/public-data/tmp/url_to_pdf/)
     * @param array $pdfOptions Liste des options de configuration dédiées au module et à la conversion html
     * @param bool $isTestMode Indique si on est en mode test, dans ce cas on ne n'insère pas les path sous forme d'url
     * @param array $domaineException Liste des domaines à ne pas traiter
     * @return string $htmlContent
     * @throws \Exception si l'image n'existe pas
     */
    public function getCleanHtml(
        string $htmlIn,
        string $projectRootPath,
        string $webPath,
        string $webPathUrl,
        array $pdfOptions = [],
        bool $isTestMode = false,
        $domaineException = []
    ): string {
        if (!$htmlIn) {
            throw new \Exception("Le contenu HTML est vide.");
        }
        if (!$projectRootPath) {
            throw new \Exception("Le chemin racine du projet n'est pas renseigné.");
        }
        if (!$webPath) {
            throw new \Exception("Le chemin vers le répertoire web n'est pas renseigné.");
        }
        if (!$webPathUrl) {
            throw new \Exception("L'url vers le répertoire web n'est pas renseigné.");
        }

        if (substr($webPath, -1) !== '/') {
            $webPath .= '/';
        }

        if (substr($webPathUrl, -1) !== '/') {
            $webPathUrl .= '/';
        }

        if (is_dir($webPath)) {
            $this->deleteDir($webPath);
        }
        if (!mkdir($webPath, 0777, true)) {
            throw new \Exception("Le répertoire web n'a pas pu être créé.");
        }

        libxml_use_internal_errors(true);
        $dom = new \DOMDocument();
        $dom->loadHTML($htmlIn);

        $tagAttributes = [
            'img' => 'src',
            'script' => 'src',
            'link' => 'href',
            'page_header' => '',
            'page_footer' => '',
            'page' => '',
        ];

        $compteurNomImage = 0;

        foreach ($tagAttributes as $tag => $attribute) {
            $elements = $dom->getElementsByTagName($tag);
            if ($elements->length === 0) {
                continue;
            }

            foreach ($elements as $element) {
                // Transforme les balises <page>, <page_header>, <page_footer> en <div>
                if (in_array($tag, ['page', 'page_header', 'page_footer'])) {
                    $newElement = $dom->createElement('div');
                    $newElement->setAttribute('id', $tag);
                    $newElement->setAttribute('class', $tag);
                    $style = $element->attributes->getNamedItem('style');
                    if ($style) {
                        $newElement->setAttribute('style', $style->nodeValue);
                    }

                    // Convertit les attributs backtop, backbottom, backleft, backright en style
                    $attributesList = $element->attributes;
                    foreach ($attributesList as $attrName => $attrNode) {
                        if (in_array($attrName, ['backtop', 'backbottom', 'backleft', 'backright'])) {
                            switch ($attrName) {
                                case 'backtop':
                                    $newElement->setAttribute('style', $newElement->getAttribute('style') . 'margin-top:' . $attrNode->nodeValue . ';');
                                    break;
                                case 'backbottom':
                                    $newElement->setAttribute('style', $newElement->getAttribute('style') . 'margin-bottom:' . $attrNode->nodeValue . ';');
                                    break;
                                case 'backleft':
                                    $newElement->setAttribute('style', $newElement->getAttribute('style') . 'margin-left:' . $attrNode->nodeValue . ';');
                                    break;
                                case 'backright':
                                    $newElement->setAttribute('style', $newElement->getAttribute('style') . 'margin-right:' . $attrNode->nodeValue . ';');
                                    break;
                            }
                        }
                    }

                    while ($element->firstChild) {
                        $newElement->appendChild($element->firstChild);
                    }
                    $element->parentNode->replaceChild($newElement, $element);
                }

                if ($attribute === '') {
                    // C'est une balise page/page_header/page_footer, déjà traitée.
                    continue;
                }

                $attributeNode = $element->attributes->getNamedItem($attribute);
                if (!$attributeNode) continue;

                $filePath = $attributeNode->nodeValue;
                if (!$filePath) continue;

                // Vérifie si c'est une URL externe
                if (filter_var($filePath, FILTER_VALIDATE_URL)) {
                    // URL externe
                    if ($this->isDomainAllowed($filePath, $domaineException)) {
                        // Domaine autorisé
                        $exist = $this->urlExists($filePath);
                        if (!$exist) {
                            throw new \Exception("Impossible d'accéder à cette image : " . $filePath);
                        }
                        // On garde l'URL telle quelle
                        if ($element instanceof \DOMElement) {
                            $element->setAttribute($attribute, $filePath);
                        }
                    } else {
                        // Domaine non autorisé, on télécharge
                        $fileExist = $this->getImageInfo($filePath);
                        $nameFileFromPath = $this->getFileName($filePath);
                        $extension = $this->getExtensionFromContentType($fileExist['content_type'], $nameFileFromPath);
                        if (!$extension) {
                            throw new \Exception("Ce type de fichier n'est pas supporté. URL: " . $filePath . " Content type: " . $fileExist['content_type']);
                        }

                        $imageData = @file_get_contents($filePath);
                        if ($imageData === false) {
                            throw new \Exception("Le fichier n'a pas pu être récupérer.");
                        }

                        $newBasename = 'url_to_pdf_image_' . $compteurNomImage . '.' . $extension;
                        $compteurNomImage++;

                        if (@file_put_contents($webPath . $newBasename, $imageData) === false) {
                            throw new \Exception("Une erreur s'est produite lors de la copie de l'image");
                        }
                        chmod($webPath . $newBasename, 0777);
                        if ($element instanceof \DOMElement) {
                            if ($element instanceof \DOMElement) {
                                $element->setAttribute($attribute, $newBasename);
                            }
                        }
                    }
                } else {
                    // Chemin local
                    $filePath = str_replace('../', '', $filePath);
                    $filePath = str_replace('./', '', $filePath);
                    $filePath = str_replace('//', '/', $filePath);

                    $formatPath = "/^(https?|ftp|s3):\/\/(localhost|[\da-z\.-]+)(:[0-9]+)?(\.[a-z\.]{2,6})?([\/\w \.-]*)*\/?$/";
                    if (preg_match($formatPath, $filePath)) {
                        // Si jamais c'était une url locale (type ftp ou s3), on récupère juste le chemin
                        $filePath = parse_url($filePath, PHP_URL_PATH);
                    }

                    $filePath = str_replace('//', '/', $filePath);

                    // Si le chemin commence par / mais n'est pas préfixé par $projectRootPath, on retire le slash
                    if (strpos($filePath, '/', 0) === 0 && strpos($filePath, $projectRootPath, 0) !== 0) {
                        $filePath = substr($filePath, 1);
                    }

                    if (realpath($filePath)) {
                        $filePath = realpath($filePath);
                    }

                    if (strpos($filePath, $projectRootPath, 0) !== 0) {
                        $filePath = rtrim($projectRootPath, '/') . '/' . ltrim($filePath, '/');
                    }

                    if (!file_exists($filePath)) {
                        throw new \Exception("Le fichier n'existe pas : " . $filePath);
                    }

                    $newBasename = 'url_to_pdf_image_' . $compteurNomImage . '.' . pathinfo($filePath, PATHINFO_EXTENSION);
                    $compteurNomImage++;

                    if ($isTestMode) {
                        $newBasename = pathinfo($filePath, PATHINFO_BASENAME);
                    }

                    $newFilePath = $webPath . $newBasename;
                    if (!copy($filePath, $newFilePath)) {
                        throw new \Exception("Le fichier n'a pas pu être copié.");
                    }
                    chmod($newFilePath, 0777);

                    // On remplace le chemin dans l'attribut de l'élément par le chemin accessible depuis l'API
                    $formatPath = "/^(https?|ftp|s3):\/\/(localhost|[\da-z\.-]+)(\.[a-z\.]{2,6})?(:\d{2,5})?([\/\w \.-]*)*\/?$/";
                    $newRelativePath = $webPathUrl . $newBasename;
                    if (!preg_match($formatPath, $newRelativePath)) {
                        throw new \Exception("L'url n'est pas valide pour : " . $newRelativePath);
                    }
                    if ($element instanceof \DOMElement) {
                        $element->setAttribute($attribute, $newBasename);
                    }
                }
            }
        }

        $htmlContent = $dom->saveHTML();

        $backgroundUrls = $this->extractAndCheckBackgroundUrls($htmlContent);
        if (substr($projectRootPath, -1) !== '/') {
            $projectRootPath .= '/';
        }

        foreach ($backgroundUrls as $backgroundImage) {
            if (filter_var($backgroundImage, FILTER_VALIDATE_URL)) {
                // URL externe
                if ($this->isDomainAllowed($backgroundImage, $domaineException)) {
                    $exist = $this->urlExists($backgroundImage);
                    if (!$exist) {
                        throw new \Exception("Impossible d'accéder à cette image : " . $backgroundImage);
                    }
                    // Domaine autorisé, on garde l'URL telle quelle
                    // On ne remplace pas l'URL par un fichier local
                    // Donc aucune modification du HTML ici pour cette URL
                    continue;
                }

                // Domaine non autorisé, on télécharge
                $fileExist = $this->getImageInfo($backgroundImage);
                $nameFileFromPath = $this->getFileName($backgroundImage);
                $extension = $this->getExtensionFromContentType($fileExist['content_type'], $nameFileFromPath);
                if (!$extension) {
                    throw new \Exception("Ce type de fichier n'est pas supporté : $backgroundImage " . $fileExist['content_type']);
                }

                $imageData = @file_get_contents($backgroundImage);
                if ($imageData === false) {
                    throw new \Exception("Le fichier n'a pas pu être récupérer.");
                }

                $newBasename = 'url_to_pdf_image_' . $compteurNomImage . '.' . $extension;
                $compteurNomImage++;

                if (@file_put_contents($webPath . $newBasename, $imageData) === false) {
                    throw new \Exception("Une erreur s'est produite lors de la copie de l'image");
                }

                chmod($webPath . $newBasename, 0777);
                $htmlContent = str_replace($backgroundImage, $newBasename, $htmlContent);
            } else {
                // Chemin local
                $localPath = str_replace('../', '', $backgroundImage);
                $localPath = str_replace('./', '', $localPath);
                $localPath = str_replace('//', '/', $localPath);

                if (strpos($localPath, '/', 0) === 0 && strpos($localPath, $projectRootPath, 0) !== 0) {
                    $localPath = substr($localPath, 1);
                }

                if (realpath($localPath)) {
                    $localPath = realpath($localPath);
                }

                if (strpos($localPath, $projectRootPath, 0) !== 0) {
                    $localPath = rtrim($projectRootPath, '/') . '/' . ltrim($localPath, '/');
                }

                if (!file_exists($localPath)) {
                    throw new \Exception("Le fichier n'existe pas : " . $localPath);
                }

                $newBasename = 'url_to_pdf_image_' . $compteurNomImage . '.' . pathinfo($localPath, PATHINFO_EXTENSION);
                $compteurNomImage++;

                if (!copy($localPath, $webPath . $newBasename)) {
                    throw new \Exception("Le fichier n'a pas pu être copié.");
                }
                chmod($webPath . $newBasename, 0777);
                $htmlContent = str_replace($backgroundImage, $newBasename, $htmlContent);
            }
        }

        $styleAttributes = '';
        if (isset($pdfOptions['noBreakTableRows']) && $pdfOptions['noBreakTableRows']) {
            $styleAttributes .= 'tr, td, th { page-break-inside: avoid; }';
        }
        if (isset($pdfOptions['font'])) {
            $styleAttributes .= 'html { ';
            foreach ($pdfOptions['font'] as $font => $att) {
                $styleAttributes .= 'font-' . $font . ': ' . $att . '; ';
            }
            $styleAttributes .= '}';
        }

        if (empty($styleAttributes)) {
            libxml_use_internal_errors(false);
            return $htmlContent;
        }

        $dom->loadHTML($htmlContent);

        if ($dom->getElementsByTagName('style')->length === 0) {
            $style = $dom->createElement('style', $styleAttributes);
        } else {
            $style = $dom->getElementsByTagName('style')->item(0);
            $style->nodeValue .= $styleAttributes;
        }

        if ($dom->getElementsByTagName('head')->length > 0) {
            $dom->getElementsByTagName('head')->item(0)->appendChild($style);
        } else {
            $head = $dom->createElement('head');
            if ($dom->getElementsByTagName('html')->item(0)->hasChildNodes()) {
                $firstChild = $dom->getElementsByTagName('html')->item(0)->firstChild;
                $dom->getElementsByTagName('html')->item(0)->insertBefore($head, $firstChild);
            } else {
                $dom->getElementsByTagName('html')->item(0)->appendChild($head);
            }
            $dom->getElementsByTagName('head')->item(0)->appendChild($style);
        }

        $htmlContent = $dom->saveHTML();

        $libxmlErrors = libxml_get_errors();
        foreach ($libxmlErrors as $error) {
            if ($error->level === LIBXML_ERR_FATAL) {
                throw new \Exception("Erreur survenue lors de la manipulation du HTML : " . $error->level . " -> " . $error->message);
            }
        }

        libxml_clear_errors();
        libxml_use_internal_errors(false);

        return $htmlContent;
    }
    /**
     * Supprime les images temporaires
     * 
     * @param string $webPath Chemin vers le répertoire web (ex: /var/www/html/public-data/tmp/url_to_pdf/)
     * @return void
     */
    public function deleteTempImages(string $webPath): void
    {
        if (substr($webPath, -1) !== '/') {
            $webPath .= '/';
        }

        if (is_dir($webPath)) {
            $this->rrmdir($webPath);
        }
    }

    private function rrmdir(string $dir): void
    {
        if (is_dir($dir)) {
            $objects = scandir($dir);
            foreach ($objects as $object) {
                if ($object != "." && $object != "..") {
                    if (is_dir($dir . "/" . $object)) {
                        $this->rrmdir($dir . "/" . $object);
                    } else {
                        unlink($dir . "/" . $object);
                    }
                }
            }
            rmdir($dir);
        }
    }

    /**
     * Supprime un dossier et son contenu m^me si non vide
     * 
     * @param string $dir Chemin vers le répertoire à supprimer
     * @return bool (true si le dossier a été supprimé)
     */
    public function deleteDir(string $dir): bool
    {
        if (!is_dir($dir)) {
            return false;
        }

        $files = array_diff(scandir($dir), ['.', '..']);
        foreach ($files as $file) {
            (is_dir("$dir/$file")) ? $this->deleteDir("$dir/$file") : unlink("$dir/$file");
        }

        if (!rmdir($dir)) {
            throw new \Exception("Le répertoire " . $dir . " n'a pas pu être supprimé.");
        }
        return true;
    }
}
